/*
 * uname() emulation by Dave Gymer. In the public domain.
 * Bugs:
 *	The MiNT version stuff is in the release field. According to the GNU
 *	shell utils this is the way SunOS does it, so we put the TOS
 *	version number in the 'version' field (even under MiNT).
 *
 * (Modified slightly by ERS.)
 *
 * -a, --all
 * -m, --machine    m68k
 * -n, --nodename   $HOSTNAME
 * -r, --release    MiNT version
 * -s, --sysname    MiNT or TOS
 * -p, --processor  _CPU cookie (68000, 68010, 68020, 68030, 68040, etc.)
 * -v                TOS version
 *
 * These options conform with GNU sh-utils 1.16 version of uname
 */

#include <stdlib.h>
#include <unistd.h>
#include <osbind.h>
#include <stdio.h>
#include <string.h>
#include <ssystem.h>
#include <mintbind.h>
#ifdef __TURBOC__
#include <sys\utsname.h>
#else
#include <sys/utsname.h>
#endif

extern int __mint;
__EXTERN int gethostname __PROTO((char *buf, size_t len));

static long _cpu;  /* value of the _CPU cookie, if any */
static int tosvers;	/* TOS version number */

/*
 * get operating system information; must execute in supervisor mode
 */

static long getinfo __PROTO((void));
static int no_ssystem;

static long
getinfo()
{
	long *cookie, *sysbase;

/* get _CPU cookie value */
	if(no_ssystem) {
		cookie = *((long **) 0x5a0L);
		if (cookie) {
			while (*cookie) {
				if (*cookie == 0x5f435055L) {	/* _CPU */
					_cpu = cookie[2];
					break;
				}
			}
		}
	}
	else
		_cpu = Ssystem(GETCOOKIE, 0x5f435055L, NULL);

/* get TOS version number */
	if (no_ssystem) {
		sysbase = *((long **)(0x4f2L));
		tosvers = (int)(sysbase[0] & 0x0000ffff);
	}
	else
		tosvers = (int)(Ssystem(TOSHEADER, 0L, NULL)&0x0000ffff);

	return 0;
}

#define HILO(x) (int) ((x >> 8) & 255), (int) (x & 255)

int
uname(buf)
	struct utsname *buf;
{
	no_ssystem = Ssystem(-1, NULL, NULL);
	if (no_ssystem) {
		if (!tosvers)
			(void)Supexec(getinfo);
	}
	else
		getinfo();

	strcpy(buf->sysname, __mint ? "MiNT" : "TOS");

	if (gethostname(buf->nodename, (size_t) sizeof(buf->nodename)))
		strcpy(buf->nodename, "??node??");
/* these should be null terminated if I'm correct... Mikko Larjava */
	buf->nodename[sizeof(buf->nodename) - 1] = '\0';

	if (__mint)
		sprintf(buf->release, "%d.%d", HILO(__mint));
	else
		buf->release[0] = 0;

	sprintf(buf->version, "%d.%d", HILO(tosvers));

	strcpy(buf->machine, "m68k");
/*
 * couldn't remember if sprintf allows a formating similar to
 * printf("680%02d", _cpu);
 * the idea is, we need to ensure that _cpu is 2 characters long,
 * even if the cookie returns a value of 0 (when we have a 68000)
 *
 */
	sprintf(buf->processor, "680%02d", _cpu);

	return 0;
}
